/* Program psd.cc       */
/* By Harald Stauss     */
/* Last change 28.02.99 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include "tsa.h"
#include "../nrc/nrutil.h" 
#include "../nrc/nrutil.cc"
#include "../nrc/four1.cc"
#include "../nrc/moment.cc"
#include "../nrc/complex.cc"

main(int argc, char *argv[])
{
  short    i;
  int      res;
  unsigned long     fn,n,eospec;
  float    ave, adev, sdev, var, skew, curt;
  float    *data;
  char     *fin,*fout,*ftin,*ftout, *pftin, *pftout;
  REGIST   r,f;
  fcomplex z;


  /* H e l l o   w o r l d */
  cout << "PSD Ver.1.1 by H. Stauss, 1999 \n";

  /* A r r a y s   a l l o c i e r e n */
  fin=(char *)malloc(255);
  fout=(char *)malloc(255);
  ftin=(char *)malloc(255);
  ftout=(char *)malloc(255);
  data=(float *)malloc(sizeof(float));

  /* P a r a m e t e r   e i n l e s e n */
  if (argc!=4)
    {
      cout << "Number of parameters wrong! \n";
      cout << "psd f_in f_out n \n";
      cout << "calculates the power spectrum based on n values \n";
      cout << "zero pads to n values if necessary, \n";
      cout << "f_in  : name of input file (*.TSA, *.ASC). \n";
      cout << "f_out : name of output file (*.TSA, *.ASC). \n";
      cout << "n     : number of data values for FFT \n";
      goto ende;
    }
  strcpy(fin,argv[1]);
  strcpy(fout,argv[2]);
  strcpy(ftin,fin);
  strcpy(ftout,fout);
  pftin=strrchr(ftin,'.');
  for (i=0; pftin[i]; i++) pftin[i]=toupper(pftin[i]);
  pftout=strrchr(ftout,'.');
  for (i=0; pftout[i]; i++) pftout[i]=toupper(pftout[i]);
  sscanf(argv[3],"%ul",&fn);

  printf("Input file  : %s \n",fin);
  printf("Output file : %s \n",fout);
  cout << "FFT on " << fn << " values \n";

  /* E i n g a b e d a t e i   o e f f n e n */
  if (strcmp(pftin,".TSA")==0) r.ReadTSA(fin);
  if (strcmp(pftin,".ASC")==0) r.ReadASCII(fin,50);
  if ((strcmp(pftin,".TSA")!=0) && (strcmp(pftin,".ASC")!=0)) {
    cout << "File type for input not recognized! \n";
    goto ende;
  }
  printf("Number of channels   : %d  \n",r.n);

  data=(float *)realloc(data, 2*fn*sizeof(float));

  /* M A I N L O O P  */
  for (i=0; i<r.n; i++) {
    cout << "n in channel " << i+1 << ": " << r.ch[i].n <<
      " FFT based on " << fn << " values ";
    if (fn<=r.ch[i].n) cout << "\n"; else cout << "=> zero padded! \n";
    // Mittelwert berechnen;
    n=fn;
    if (r.ch[i].n<fn) n=r.ch[i].n;
    moment(r.ch[i].data-1,n,&ave,&adev,&sdev,&var,&skew,&curt);
    /* Daten fuer FFT kopieren */
    for (n=0; n<fn; n++) {
      if (n<r.ch[i].n) { 
	data[n*2]=r.ch[i].data[n]-ave;
	data[n*2+1]=0;
      }
      else {
	data[n*2]=0;
	data[n*2+1]=0;
      }
    }
    four1(data-1,fn,1);
    // in neue REGIST copieren;
    f.AddChannel(fn/2+1, r.ch[0].sr, 0, 1, 0, 0,"fft", "fft");
    z.r=data[0]; z.i=data[1];
    f.ch[i].data[0]=Cabs(z)*Cabs(z)/fn/fn;
    for (n=1; n<fn/2; n++) {
      z.r=data[n*2]; z.i=data[n*2+1];
      f.ch[i].data[n]=2*Cabs(z)*Cabs(z)/fn/fn;
    } /* endfor */
    z.r=data[fn];z.i=data[fn+1];
    f.ch[i].data[fn/2]=Cabs(z)*Cabs(z)/fn/fn;
  } /* endfor */

  /* D a t e i   s c h r e i b e n */
  if (strcmp(pftout,".TSA")==0) f.WriteTSA(fout);
  if (strcmp(pftout,".ASC")==0) f.WriteASCII(fout);
  if ((strcmp(pftout,".TSA")!=0) && (strcmp(pftout,".ASC")!=0)) {
    printf("File type for output not recognized! \n");
    goto ende;
  }

 ende:
  free(data);free(fin);free(ftin);free(fout);free(ftout);
  return(0);
} /* Ende von main */
